//
//  CalcOperation.m
//  MandelOpper
//
//  Created by Ken Anderson on 4/29/12.
//  Copyright (c) 2012 University of Colorado, Boulder. All rights reserved.
//

#import "CalcOperation.h"

#import "Bitmap.h"
#import "BitmapView.h"

#import <complex.h>

#define LOOP 500
#define LIMIT 256

@implementation CalcOperation

void gradient (int value, unsigned char *buffer) {
    unsigned char *ptr = buffer;
    value = value * 4;
    if (value > 255) {
        value = 255;
    }
    *ptr++ = value / 3;
    *ptr++ = value / 2;
    *ptr++ = value;
    *ptr++ = 0xFF;
}

void mandelbrot(double x, double y, unsigned char *buffer) {
    int i;
    complex double z, c;
    
    c = x + (y * 1.0i);
    z = 0;
    
    for (i = 0; i < LOOP; i++) {
        z = (z * z) + c;
        if (cabs(z) > LIMIT) {
            gradient(i, buffer);
            return;
        }
    }
    gradient(0, buffer);
}

- (id) initWithBitmap: (Bitmap *) bitmap
           bitmapView: (BitmapView *) view
        calculateLine: (NSUInteger) line
               xStart: (double) xStart
                 xEnd: (double) xEnd
                    y: (double) y {
    if ((self = [super init])) {
        _bitmap = bitmap;
        _view = view;
        _line = line;
        _xStart = xStart;
        _xEnd = xEnd;
        _y = y;
    }
    return self;
}

- (void) main {
    if ([self isCancelled]) {
        return;
    }
    
//    int usec = random() % 1000;
//    usec *= 3000;
//    usleep(usec);
  
    unsigned char *scanline = [_bitmap baseAddressForLine:_line];
    double xDelta = (_xEnd - _xStart) / _bitmap.width;
    double x = _xStart;
    
    for (int i = 0; i < _bitmap.width; i++) {
        unsigned char *pixel = (scanline + i * 4);
        x += xDelta;
        mandelbrot(x, _y, pixel);
        
        if (i % 50 == 0) {
            if ([self isCancelled]) {
                return;
            }
        }
    }
    
    [_view performSelectorOnMainThread: @selector(bnr_setNeedsDisplay) withObject: nil waitUntilDone:NO];
}



@end
